package com.synnex.notification.core;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.AsyncContext;
import javax.servlet.http.HttpServletResponse;

import org.apache.juli.logging.Log;
import org.apache.juli.logging.LogFactory;

import com.synnex.notification.util.StringUtil;
import com.synnex.notification.vo.User;

public class ClientThread implements Runnable
{
	private static final Log log = LogFactory.getLog(ClientThread.class);
	
	private AsyncContext asyncContext;
	private User user;
	
	private List<String> msgList = new ArrayList<String>();
	private boolean isAsyncCompleted;
	private boolean logout;
	
	public ClientThread(User user, AsyncContext asyncContext)
	{
		this.user = user;
		this.asyncContext = asyncContext;
		asyncContext.setTimeout(Config.CLIENT_THREAD_MAX_TIMEOUT);
	}
	public void run()
	{
		try
		{
			while(!isAsyncCompleted)
			{
				writeToClient();
				if(logout)
				{
					log.info("## user logout, user name: " + user.getUsername() + " ##");
					asyncCompleted();
				}
				else
				{
					sleep();
				}
			}
		}
		catch (Exception e)
		{
			log.error("client thread run error: ", e);
		}
		log.info("## client thread completed ##");
	}
	public void start()
	{
		Thread thread = new Thread(this);
		thread.setName("ClientThread-" + user.getUsername());
		thread.start();
		log.info("## client thread start");
	}
	public void sendMessage(String msg)
	{
		msgList.add(msg);
		wake();
	}
	public void stop()
	{
		asyncCompleted();
		wake();
	}
	private synchronized void sleep() throws InterruptedException
	{
		wait();
	}
	private synchronized void wake()
	{
		notify();
	}
	private void writeToClient()
	{
		if(msgList.size() <= 0)
		{
			log.info("## message is empty or HttpServletResponse is null");
			return;
		}
		try
		{
			HttpServletResponse response = (HttpServletResponse)asyncContext.getResponse();
			PrintWriter writer = response.getWriter();
			for(String msg : msgList)
			{
				if(!StringUtil.isEmpty(msg))
				{
					writer.write(msg);
				}
				if(Config.MSG_LOGOUT.equals(msg))
				{
					logout = true;
				}
			}
			writer.flush();
			response.flushBuffer();
			log.info("## send message to " + user.getUsername() + " success !");
		}
		catch (Exception e)
		{
			log.error("## client thread write message to client error: ", e);
			stop();
		}
		msgList.clear();
	}
	private void asyncCompleted()
	{
		try
		{
			asyncContext.complete();
		}
		catch (Exception e)
		{
			log.error("Async Context completed error: ", e);
		}
		isAsyncCompleted = true;
	}
	public boolean isAsyncCompleted()
	{
		return isAsyncCompleted;
	}
}