package com.synnex.notification.core;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import javax.servlet.AsyncContext;

import org.apache.juli.logging.Log;
import org.apache.juli.logging.LogFactory;

import com.synnex.notification.util.StringUtil;
import com.synnex.notification.vo.User;

public class NotificationManager
{
	private NotificationManager(){}
	
	private static final Log log = LogFactory.getLog(NotificationManager.class);

	private static Map<User, ClientThread> map = new HashMap<User, ClientThread>();
	
	static
	{
		sendHeartbeat();
	}
	
	public static void addClient(User user, AsyncContext asyncContext)
	{
		if(user == null || asyncContext == null)
		{
			return;
		}
		ClientThread clientThread = new ClientThread(user, asyncContext);
		ClientThread oldClientThread = map.put(user, clientThread);
		clientThread.start();
		if(oldClientThread != null)
		{
			oldClientThread.stop();
		}
	}
	/**
	 * send message to all user
	 */
	public static void sendMessage(String msg)
	{
		if(StringUtil.isEmpty(msg))
		{
			return;
		}
		if(map.size() <= 0)
		{
			return;
		}
		clearClient();
		Set<User> userList = map.keySet();
		for(User user : userList)
		{
			sendMessage(user, msg);
		}
	}
	public static void sendMessage(User user, String msg)
	{
		if(user == null || StringUtil.isEmpty(msg))
		{
			return;
		}
		ClientThread clientThread = map.get(user);
		if(clientThread == null)
		{
			log.error("client thread not found");
			return;
		}
		clientThread.sendMessage(msg);
		clearClient();
	}
	
	public static Set<User> getAllUser()
	{
		return map.keySet();
	}
	
	public static void logout(User user)
	{
		if(user == null)
		{
			return;
		}
		ClientThread clientThread = map.get(user);
		clientThread.stop();
	}
	private static void sendHeartbeat()
	{
		Timer timer = new Timer();
		timer.schedule(new TimerTask()
		{
			public void run()
			{
				sendMessage("## this is the heartbeat message ##");
			}
		}, new Date(), Config.HEART_BEAT_INTERVAL);
	}
	private static void clearClient()
	{
		// remove client thread if client logout
		Map<User, ClientThread> tmpMap = new HashMap<User, ClientThread>();
		for(Map.Entry<User, ClientThread> entry : map.entrySet())
		{
			ClientThread clientThread = entry.getValue();
			if(clientThread != null && !clientThread.isAsyncCompleted())
			{
				tmpMap.put(entry.getKey(), entry.getValue());
			}
			else if(clientThread != null && clientThread.isAsyncCompleted() && entry.getKey() != null)
			{
				log.info("## " + entry.getKey().getUsername() + " logout ##");
			}
		}
		map.clear();
		map.putAll(tmpMap);
	}
	public static void removeClient(User user)
	{
		if(user == null)
		{
			return;
		}
		ClientThread clientThread = map.get(user);
		if(clientThread == null)
		{
			return;
		}
		clientThread.sendMessage(Config.MSG_LOGOUT);
		map.remove(user);
	}
}