package com.synnex.notification.servlet;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.AsyncContext;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.juli.logging.Log;
import org.apache.juli.logging.LogFactory;

import com.synnex.notification.core.Config;
import com.synnex.notification.core.NotificationManager;
import com.synnex.notification.util.StringUtil;
import com.synnex.notification.vo.User;

@WebServlet(name="NotificationServlet", urlPatterns={"/notification.do"}, asyncSupported=true)
public class NotificationServlet extends HttpServlet
{
	private static final long serialVersionUID = -2931231860618864009L;
	private static final Log log = LogFactory.getLog(NotificationServlet.class);
	
	public NotificationServlet()
	{
		log.info("## Notification Servlet created ##");
	}
	
	protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
	{
		response.setContentType("application/json");
		String username = request.getParameter(Config.PARAM_USERNAME);
		String action = request.getParameter(Config.PARAM_ACTION);
		
		if(StringUtil.isEmpty(username))
		{
			log.info("## username is empty");
			writeToClient(response, "please pass username to server side.");
			return;
		}
		User user = new User(username);
		if("logout".equals(action))
		{
			NotificationManager.removeClient(user);
		}
		else
		{
			AsyncContext asyncContext = request.startAsync();
			NotificationManager.addClient(user, asyncContext);
		}
	}
	private void writeToClient(HttpServletResponse response, String msg)
	{
		if(StringUtil.isEmpty(msg) || response == null)
		{
			log.info("## message is empty or HttpServletResponse is null");
			return;
		}
		try
		{
			PrintWriter writer = response.getWriter();
			writer.write(msg);
			writer.flush();
			response.flushBuffer();
		}
		catch (Exception e)
		{
			log.error("## write message to client error: ", e);
		}
	}
}
